@@ -7,7 +7,6 @@ from sqlalchemy_api_handler.bases.accessor import Accessor
 from sqlalchemy_api_handler.bases.errors import ActivityError
 from sqlalchemy_api_handler.bases.save import Save
 from sqlalchemy_api_handler.utils.datum import relationships_in
-from sqlalchemy_api_handler.utils.logger import logger
 
 
 def merged_datum_from_activities(activities,
@@ -33,7 +32,8 @@ class Activator(Save):
                  with_check_not_soft_deleted=True):
         Activity = Activator.get_activity()
         for (entity_identifier, grouped_activities) in groupby(activities, key=lambda activity: activity.entityIdentifier):
-            grouped_activities = sorted(grouped_activities, key=lambda activity: activity.dateCreated)
+            grouped_activities = sorted(grouped_activities,
+                                        key=lambda activity: activity.dateCreated)
 
             first_activity = grouped_activities[0]
             table_name = first_activity.table_name
@@ -121,13 +121,12 @@ class Activator(Save):
 
             db = Activator.get_db()
             db.session.add_all(grouped_activities)
-            table_name = model.__tablename__
-            db.session.execute(f'ALTER TABLE {table_name} DISABLE TRIGGER audit_trigger_update;')
+            db.session.execute(f'ALTER TABLE {model.__tablename__} DISABLE TRIGGER audit_trigger_update;')
             entity.modify(merged_datum,
                           with_add=True,
                           with_check_not_soft_deleted=with_check_not_soft_deleted)
             db.session.flush()
-            db.session.execute(f'ALTER TABLE {table_name} ENABLE TRIGGER audit_trigger_update;')
+            db.session.execute(f'ALTER TABLE {model.__tablename__} ENABLE TRIGGER audit_trigger_update;')
             db.session.commit()
 
 
