@@ -1,19 +1,23 @@
 import binascii
 from base64 import b32encode, b32decode
+from typing import Any
 
 from sqlalchemy_api_handler.utils.is_id_column import is_id_column
-# This library creates IDs for use in our URLs,
-# trying to achieve a balance between having a short
-# length and being usable by humans
-# We use base32, but replace O and I, which can be mistaken for 0 and 1
-# by 8 and 9
+
 
 class NonDehumanizableId(Exception):
     pass
 
 
 def dehumanize(publicId):
-    """ Get back an integer from a human-compatible ID """
+    """
+    Get back an integer from a human-compatible ID
+    This library creates IDs for use in our URLs,
+    trying to achieve a balance between having a short
+    length and being usable by humans
+    We use base32, but replace O and I, which can be mistaken for 0 and 1
+    by 8 and 9
+    """
     if publicId is None:
         return None
     missing_padding = len(publicId) % 8
@@ -39,3 +43,9 @@ def dehumanize_ids_in(datum, model):
             if is_id_column(getattr(model, key)):
                 dehumanized_datum[key] = dehumanize(value)
     return dehumanized_datum
+
+
+def dehumanize_if_needed(column, value: Any) -> Any:
+    if is_id_column(column):
+        return dehumanize(value)
+    return value
